#ifndef B_INTERFACE
#define B_INTERFACE

#include "SDL_ttf.h"

class BInterfaceElement {
public:
	BInterfaceElement();
	virtual ~BInterfaceElement();

	virtual bool check_status(int mousex,int mousey,int button,int button_status,class KEYBOARDSTATE *k);
	virtual void draw(float alpha);
	virtual void draw(void);

	int m_ID;
	bool m_modal;	/* If any element is modal, only him has the control until it is destroyed (the rest of the interface is faded) */
	bool m_enabled;
	bool m_active;	/* This indictes wether the component is active or passive, */
					/* e.g.: BText and Bframe are passive					*/
	bool m_to_be_deleted;

	float m_x,m_y,m_dx,m_dy;
};


class BText : public BInterfaceElement {
public:

	BText(char *text,TTF_Font *font,float x,float y,bool centered);
	BText(char *text,TTF_Font *font,float x,float y,bool centered,int ID);
	virtual ~BText();

	virtual void draw(float alpha);
	virtual void draw(void);

	void set_text(char *text);

	bool m_centered;
	char *m_text;
	TTF_Font *m_font;
};


class Bbutton : public BInterfaceElement {
public:

	Bbutton(char *text,TTF_Font *font,float x,float y,float dx,float dy,int ID);
	Bbutton(GLTile *icon,float x,float y,float dx,float dy,int ID);
	virtual ~Bbutton();

	virtual bool check_status(int mousex,int mousey,int button,int button_status,KEYBOARDSTATE *k);
	virtual void draw(float alpha);
	virtual void draw(void);

	char *m_text;
	TTF_Font *m_font;
	int m_status;
	GLTile *m_tile;
};


class Bframe : public BInterfaceElement {
public:

	Bframe(float x,float y,float dx,float dy);
	virtual ~Bframe();

	virtual void draw(float alpha);
	virtual void draw(void);

};


class BTextInputFrame : public Bframe {
public:

	BTextInputFrame(char *initial_text,int max_characters,TTF_Font *font,float x,float y,float dx,float dy,int ID);
	virtual ~BTextInputFrame();

	virtual bool check_status(int mousex,int mousey,int button,int button_status,KEYBOARDSTATE *k);
	virtual void draw(float alpha);
	virtual void draw(void);

	TTF_Font *m_font;
	char *m_editing;
	int m_max_characters;
	unsigned int m_editing_position;
	bool m_focus;
	int m_cycle;
	bool m_change_in_last_cycle;
};


class Bslider : public BInterfaceElement {
public:

	Bslider(float x,float y,float dx,float dy,float slider_dx,float slider_dy,int ID);
	virtual ~Bslider();

	virtual bool check_status(int mousex,int mousey,int button,int button_status,KEYBOARDSTATE *k);
	virtual void draw(float alpha);
	virtual void draw(void);

	float m_slider_dx,m_slider_dy;
	float m_value;
};


class BBrowser : public BInterfaceElement {
public:

	BBrowser(TTF_Font *font,float x,float y,float dx,float dy,int ID);
	virtual ~BBrowser();

	void clear(void);
	void addEntry(char *);
	char *getEntry(int i);
	void deleteEntry(int i);
	void setSelected(int i);
	int getSelected(void);

	virtual bool check_status(int mousex,int mousey,int button,int button_status,KEYBOARDSTATE *k);
	virtual void draw(float alpha);
	virtual void draw(void);

	TTF_Font *m_font;
	List<char> m_entries;
	float m_slider_pos,m_slider_height;
	int m_selected,m_mouse_over;
	int m_old_mouse_x,m_old_mouse_y;

};

/*
class BTileBrowser : public BInterfaceElement {
public:

	BTileBrowser(float x,float y,float dx,float dy,int ID);
	virtual ~BTileBrowser();

	void clear(void);
	void addEntry(GLTile *tile);
	GLTile *getEntry(int i);
	void deleteEntry(int i);
	void setSelected(int i);
	int getSelected(void);

	virtual bool check_status(int mousex,int mousey,int button,int button_status,KEYBOARDSTATE *k);
	virtual void draw(float alpha);
	virtual void draw(void);

	List<GLTile> m_entries;
	float m_slider_pos,m_slider_height;
	int m_selected,m_mouse_over;

};
*/


class BConfirmation : public BInterfaceElement {
public:

	BConfirmation(char *message,TTF_Font *font,float x,float y,int ID);
	virtual ~BConfirmation();

	virtual bool check_status(int mousex,int mousey,int button,int button_status,KEYBOARDSTATE *k);
	virtual void draw(float alpha);
	virtual void draw(void);

	TTF_Font *m_font;
	char *m_message;
	int m_state,m_cycle;
	Bbutton *m_ok_button,*m_cancel_button;
};





class Binterface {
public:
	static List<BInterfaceElement> m_elements;

	static void add_element(BInterfaceElement *b);
	static void remove_element(BInterfaceElement *b);
	static void remove_element(int ID);
	static void substitute_element(BInterfaceElement *old,BInterfaceElement *n);
	static void reset(void);
	static int update_state(int mousex,int mousey,int button,int button_status,KEYBOARDSTATE *k);
	static void draw(float alpha);
	static void draw(void);
	static BInterfaceElement *get(int ID);
	static void enable(int ID);
	static void disable(int ID);
	static bool is_enabled(int ID);

	static void clear_print_cache(void);
	static void print_left(char *text,TTF_Font *font,float x,float y);
	static void print_center(char *text,TTF_Font *font,float x,float y);
	static void print_left(char *text,TTF_Font *font,float x,float y,float r,float g,float b,float a);
	static void print_center(char *text,TTF_Font *font,float x,float y,float r,float g,float b,float a);

	static void string_dimensions(char *text,TTF_Font *font,int *dx,int *dy);

	static void string_editor_cycle(char *editing_text,unsigned int *editing_position,unsigned int max_length,KEYBOARDSTATE *k);
};

#endif
