#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "math.h"
#include "string.h"

#include "GL/gl.h"
#include "SDL.h"
#include "SDL_mixer.h"

#include "BList.h"
#include "Symbol.h"
#include "2DCMC.h"
#include "auxiliar.h"
#include "GLTile.h"
#include "2DCMC.h"
#include "keyboardstate.h"
#include "randomc.h"
#include "VirtualController.h"

#include "GLTManager.h"
#include "SoundManager.h"
#include "SFXManager.h"
#include "MusicManager.h"
#include "KObject.h"
#include "Knightmare.h"
#include "KnightmareApp.h"

#include "KnightmareCtnt.h"

#include "KObjectCreator.h"

#include "Popolon.h"

// #include "font_extractor.h"

#include "debug.h"

#define MAPPAUSE_TIME 25

extern bool ambient_light;

Knightmare::Knightmare(int level, int score, int sfx_volume,
                       int music_volume, GLTManager *GLTM, SFXManager *SFXM)
{	
	m_score = score;
	
	m_font16=TTF_OpenFont("fonts/OLYMRG__.TTF",16);
	
    m_sfx_volume = sfx_volume;
    m_music_volume = music_volume;

    m_cycle = 0;

    m_gamepaused = PAUSESTATE_PLAYING;
    m_gamepaused_timmer = 0;

    m_current_zoom = 1.f;
	m_camera_angle = 0.0f;

    switch(level) {
    case 1: m_map = new KMap("maps/Stage1.tmx","maps/enemy-waves-stage1.xml");
			break;
    case 2: m_map = new KMap("maps/Stage2.tmx","maps/enemy-waves-stage1.xml");
			break;
    case 3: m_map = new KMap("maps/Stage3.tmx","maps/enemy-waves-stage1.xml");
			break;
    case 4: m_map = new KMap("maps/Stage4.tmx","maps/enemy-waves-stage1.xml");
			break;
    case 5: m_map = new KMap("maps/Stage5.tmx","maps/enemy-waves-stage1.xml");
			break;
    case 6: m_map = new KMap("maps/Stage6.tmx","maps/enemy-waves-stage1.xml");
			break;
    case 7: m_map = new KMap("maps/Stage7.tmx","maps/enemy-waves-stage1.xml");
			break;
    case 8: m_map = new KMap("maps/Stage8.tmx","maps/enemy-waves-stage1.xml");
			break;
    }

	output_debug_message("m_map_position will be: %i - %i = %i!\n",m_map->m_height,SCREEN_Y,m_map->m_height-SCREEN_Y);
	
	// Add all the obstacle classes:
	obstacle_classes.Add(columncomplete_symbol);
	obstacle_classes.Add(column_symbol);

    // Add Popolon and Galious:
    m_map->addObject(4,new Popolon(m_map->m_width/2-10,m_map->m_height-160,sfx_volume,GLTM,SFXM,0));

//    m_map_position = 200;
}


Knightmare::~Knightmare()
{
	TTF_CloseFont(m_font16);
	
	obstacle_classes.ExtractAll();
	delete m_map;
	m_map = 0;
}


void Knightmare::clear(void)
{
}


bool Knightmare::cycle(List<class VirtualController> *vcl, GLTManager *GLTM, SFXManager *SFXM, MusicManager *MusicM)
{
	if (m_map!=0) m_map->cycle(vcl,this,GLTM,SFXM);
    return true;
}

void Knightmare::draw(GLTManager *GLTM)
{
	glClearStencil(0);

    m_map->draw(0,0,m_current_zoom,m_camera_angle,GLTM,m_preserve_angle);
}


void Knightmare::set_music_volume(int v, MusicManager *MusicM)
{
    m_music_volume = v;
	MusicM->music_volume(MUSIC_ID,v);
}


void Knightmare::set_sfx_volume(int v)
{
    m_sfx_volume = v;
}


bool Knightmare::game_paused()
{
	return (m_gamepaused == PAUSESTATE_PAUSING || m_gamepaused == PAUSESTATE_PAUSED);
} /* Knightmare::game_paused */

