#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include "windows.h"
#endif

#include "math.h"

#include "Lightning.h"


Lightning::Lightning(float startx,float starty)
{
	// Simple Lightning:
	int n_points = 10;
	int i;
	float prev_x = startx;
	float prev_y = starty;
	float prev_w = 1.0f;
	float x,y,w;

	for(i=0;i<n_points;i++) {
		x = prev_x + (rand()%64)-32;
		y = prev_y + 16 + (rand()&40);
		w = prev_w *0.8f;

		m_x1.Add(new float(prev_x));
		m_y1.Add(new float(prev_y));
		m_w1.Add(new float(prev_w));
		m_x2.Add(new float(x));
		m_y2.Add(new float(y));
		m_w2.Add(new float(w));

		// Create some appendixes:
		if ((rand()%2)==0) {
			m_x1.Add(new float(prev_x));
			m_y1.Add(new float(prev_y));
			m_w1.Add(new float(prev_w));
			m_x2.Add(new float(prev_x-(x-prev_x)));
			m_y2.Add(new float(prev_y + 16 + (rand()&40)));
			m_w2.Add(new float(w));
		} // if 

		prev_x = x;
		prev_y = y;
		prev_w = w;
	} // for	
	
	// split each line in two (brownian motion)
	for(i=0;i<2;i++) {
		m_x1.Rewind();
		m_y1.Rewind();
		m_w1.Rewind();
		m_x2.Rewind();
		m_y2.Rewind();
		m_w2.Rewind();
		
		int l = m_x1.Length();
		float *x1,*y1,*w1,*x2,*y2,*w2;
		for(int j = 0;j<l;j++) {
			m_x1.Iterate(x1);
			m_y1.Iterate(y1);
			m_w1.Iterate(w1);
			m_x2.Iterate(x2);
			m_y2.Iterate(y2);
			m_w2.Iterate(w2);
			
			float x3,y3,w3;
			float r1 = ((rand()%65)-32)/32.0;
			float r2 = ((rand()%65)-32)/32.0;
			float dx = fabs(*x2 - *x1);
			float dy = fabs(*y2 - *y1);
			float d = (dx>dy ? dx:dy);
			x3 = (*x1 + *x2)/2 + ((d+dx)*r1*0.2f);
			y3 = (*y1 + *y2)/2 + ((d+dy)*r2*0.2f);
			w3 = (*w1 + *w2)/2;
			
			m_x1.Add(new float(x3));
			m_y1.Add(new float(y3));
			m_w1.Add(new float(w3));
			m_x2.Add(new float(*x2));
			m_y2.Add(new float(*y2));
			m_w2.Add(new float(*w2));
			
			*x2 = x3;
			*y2 = y3;
			*w2 = w3;			
		}
	}

} // Lightning::Lightning


void Lightning::draw(float r,float g,float b,float a)
{
	float *x,*y,*w;
//	GLboolean smoothState = glIsEnabled(GL_LINE_SMOOTH);
//	glEnable(GL_LINE_SMOOTH);
	glBegin(GL_QUADS);
	m_x1.Rewind();
	m_y1.Rewind();
	m_w1.Rewind();
	m_x2.Rewind();
	m_y2.Rewind();
	m_w2.Rewind();

	while(m_x1.Iterate(x)) {
		m_y1.Iterate(y);
		m_w1.Iterate(w);

//		glColor4f(r,g,b,a*(*w));
		glColor4f(r,g,b,a);
		glVertex3f((*x)-2.0f*(*w),*y,0);
		glVertex3f((*x)+2.0f*(*w),*y,0);

		m_x2.Iterate(x);
		m_y2.Iterate(y);
		m_w2.Iterate(w);

//		glColor4f(r,g,b,a*(*w));
		glVertex3f((*x)+2.0f*(*w),*y,0);
		glVertex3f((*x)-2.0f*(*w),*y,0);
	} // while 
	glEnd();

	glPointSize(1.0f);
//	if (!smoothState) glDisable(GL_LINE_SMOOTH);

} // Lightning::draw


